from lib.common.genome.genome_feature2 import read_gff_file
from api.xuyuxing.file_parser.fileIO import tsv_file_parse
from api.xuyuxing.genome.repeatmasker import repeatmasker_parser
from lib.common.math.interval import merge_intervals, overlap_between_interval_set, sum_interval_length
from lib.common.genome.seq_base import sequence_entropy, read_fasta_by_faidx
from interlap import InterLap


def AnnoCleaner_main(args):
    gene_info = read_gff_file(args.gene_gff)
    gene_info = gene_info['gene']

    # entropy, nr and interpro
    nr_info = {}
    if args.nr_bls is not None:
        nr_info_raw = tsv_file_parse(args.nr_bls)
        for i in nr_info_raw:
            nr_info[nr_info_raw[i][0]] = 0

    interpro_info = {}
    if args.interpro_tsv is not None:
        interpro_info_raw = tsv_file_parse(args.interpro_tsv)
        for i in interpro_info_raw:
            interpro_info[interpro_info_raw[i][0]] = 0

    entropy_info = {}
    if args.entropy_tsv is not None:
        entropy_raw = tsv_file_parse(args.entropy_tsv)
        for i in entropy_raw:
            gene_id = entropy_raw[i][0]
            entropy = float(entropy_raw[i][1])
            entropy_info[gene_id] = entropy

    # bad repeat
    bad_repeat_dict = {}
    if args.repeat_gff_file is not None:
        for gene_id in gene_info:
            gf = gene_info[gene_id]
            bad_repeat_dict[(gf.chr_loci.chr_id, gf.chr_loci.strand)] = []

        repeat_gff = read_gff_file(args.repeat_gff_file)
        for i in repeat_gff.keys():
            for gf_id in repeat_gff[i]:
                gf = repeat_gff[i][gf_id]
                if gf.strand is None:
                    for strand_now in ['+', '-']:
                        if (gf.chr_id, strand_now) in bad_repeat_dict:
                            bad_repeat_dict[(gf.chr_id, strand_now)].append(
                                (gf.start, gf.end))
                else:
                    if (gf.chr_id, gf.strand) in bad_repeat_dict:
                        bad_repeat_dict[(gf.chr_id, gf.strand)].append(
                            (gf.start, gf.end))

        for i in bad_repeat_dict:
            merged_intervals = merge_intervals(bad_repeat_dict[i], True)
            bad_repeat_dict[i] = InterLap()
            if len(merged_intervals) > 0:
                bad_repeat_dict[i].update(merged_intervals)

    # tran
    tran_evidence_dict = {}
    if args.tran_evidence_gff is not None:
        for gene_id in gene_info:
            gf = gene_info[gene_id]
            tran_evidence_dict[(gf.chr_loci.chr_id, gf.chr_loci.strand)] = []

        # tran
        tran_info_raw = tsv_file_parse(args.tran_evidence_gff)
        for i in tran_info_raw:
            q_name, strand, start, end = tran_info_raw[i][0], tran_info_raw[i][6], int(
                tran_info_raw[i][3]), int(tran_info_raw[i][4])
            if (q_name, strand) in bad_repeat_dict:
                tran_evidence_dict[(q_name, strand)].append((start, end))

        for i in tran_evidence_dict:
            merged_intervals = merge_intervals(tran_evidence_dict[i], True)
            tran_evidence_dict[i] = InterLap()
            if len(merged_intervals) > 0:
                tran_evidence_dict[i].update(merged_intervals)

    # judge

    with open(args.output_file, 'w') as f:
        f.write(
            "gene_id\tmRNA_id\ttran_ratio\tbad_repeat_ratio\tentropy\tnr_anno\tinterpro_anno\tgood_gene\n")
        for gene_id in gene_info:
            gf = gene_info[gene_id]
            contig = (gf.chr_loci.chr_id, gf.chr_loci.strand)

            for mRNA in gf.sub_features:

                exon_intervals = [(i.start, i.end)
                                  for i in mRNA.sub_features if i.type == 'exon']
                if len(exon_intervals) == 0:
                    exon_intervals = [(i.start, i.end)
                                      for i in mRNA.sub_features if i.type == 'CDS']

                if contig in tran_evidence_dict:
                    tran_intervals = list(
                        tran_evidence_dict[contig].find((gf.start, gf.end)))
                else:
                    tran_intervals = []
                overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
                    exon_intervals, tran_intervals)
                tran_overlap_ratio = overlap_length / \
                    sum_interval_length(exon_intervals)

                if contig in bad_repeat_dict:
                    bad_repeat_intervals = list(
                        bad_repeat_dict[contig].find((gf.start, gf.end)))
                else:
                    bad_repeat_intervals = []
                overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
                    exon_intervals, bad_repeat_intervals)
                bad_repeat_overlap_ratio = overlap_length / \
                    sum_interval_length(exon_intervals)

                nr_anno = gene_id in nr_info or mRNA.id in nr_info
                interpro_anno = gene_id in interpro_info or mRNA.id in interpro_info

                if gene_id in entropy_info:
                    entropy = entropy_info[gene_id]
                elif mRNA.id in entropy_info:
                    entropy = entropy_info[mRNA.id]
                else:
                    continue

                good_gene = False
                if tran_overlap_ratio >= args.tran_threshold or nr_anno or interpro_anno:
                    good_gene = True
                if bad_repeat_overlap_ratio >= args.repeat_threshold:
                    good_gene = False
                if entropy <= args.entropy_threshold:
                    good_gene = False

                if good_gene:
                    f.write("%s\t%s\t%.2f\t%.2f\t%.4f\t%s\t%s\t%s\n" % (gene_id, mRNA.id, tran_overlap_ratio,
                                                                        bad_repeat_overlap_ratio, entropy, str(nr_anno), str(interpro_anno), str(good_gene)))


if __name__ == "__main__":
    # for Ocu
    class abc():
        pass

    args = abc()

    args.gene_gff = '/lustre/home/xuyuxing/Database/Orobanche/annotation/pasa/sample_mydb_pasa.gene_structures_post_PASA_updates.9505.gff3'
    # qx -pe smp 56 diamond blastp -q Gel.gene_model.v3.0.protein.fasta -k 50 -d ~/Database/NCBI/nr/2020/nr.dmnd -o Gel.gene_model.v3.0.protein.nr.bls -f 6 -p 56
    args.nr_bls = '/lustre/home/xuyuxing/Database/Orobanche/annotation/filter/pasa.aa.nr.bls'
    # qx -pe smp 20 -l h=cn01 /lustre/home/xuyuxing/Database/Interpro/interproscan-5.45-80.0/interproscan.sh -i Gel.gene_model.v3.0.protein.fasta -f tsv -b Gel.gene_model.v3.0.protein.interpro -iprlookup --goterms --pathways -dp
    args.interpro_tsv = '/lustre/home/xuyuxing/Database/Orobanche/annotation/filter/pasa.aa.interpro.tsv'
    args.repeat_gff_file = '/lustre/home/xuyuxing/Database/Orobanche/annotation/EDTA/Ocu.genome.nt.1.0.fasta.mod.MAKER.masked.gff3'
    args.tran_evidence_gff = '/lustre/home/xuyuxing/Database/Orobanche/annotation/evm/T.gff3'
    # python ~/python_project/Genome_work_tools/SeqParser.py SeqEntropy Gel.gene_model.protein.fasta Gel.gene_model.protein.entropy.txt
    args.entropy_tsv = '/lustre/home/xuyuxing/Database/Orobanche/annotation/filter/pasa.aa.entropy.txt'
    args.output_file = '/lustre/home/xuyuxing/Database/Orobanche/annotation/filter/pasa.AnnoCleaner.out'

    args.tran_threshold = 0.5
    args.repeat_threshold = 0.5
    args.entropy_threshold = 3.9

    # for Gel

    class abc():
        pass

    args = abc()

    args.gene_gff = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T99112N0.genome.gff3'
    # qx -pe smp 56 diamond blastp -q Gel.gene_model.v3.0.protein.fasta -k 50 -d ~/Database/NCBI/nr/2020/nr.dmnd -o Gel.gene_model.v3.0.protein.nr.bls -f 6 -p 56
    args.nr_bls = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T99112N0.gene_model.protein.nr.bls'
    # qx -pe smp 20 -l h=cn01 /lustre/home/xuyuxing/Database/Interpro/interproscan-5.45-80.0/interproscan.sh -i Gel.gene_model.v3.0.protein.fasta -f tsv -b Gel.gene_model.v3.0.protein.interpro -iprlookup --goterms --pathways -dp
    args.interpro_tsv = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T99112N0.gene_model.protein.interpro.tsv'
    args.repeatmasker_out = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/RepeatMasker.out/T99112N0.genome.fasta.out'
    args.tran_evidence_gff = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T.gff3'
    # python ~/python_project/Genome_work_tools/SeqParser.py SeqEntropy Gel.gene_model.protein.fasta Gel.gene_model.protein.entropy.txt
    args.entropy_tsv = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/T99112N0.gene_model.entropy.txt'
    args.output_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Phelipanche_aegyptiaca/no_clean/gene.clean.txt'

    args.tran_threshold = 0.5
    args.repeat_threshold = 0.5
    args.entropy_threshold = 3.9

    AnnoCleaner_main(args)

    # retry
    Gelv2_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/jbrowse/data/Gel.genome.gff3'
    Yuan_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/yuan_pt_vs_xu/WPG_working/WPGmapper/yuan/genewise.best.gff'

    from lib.common.genome.genome_feature2 import read_gff_file, get_mRNA_overlap, Gene, gene_compare, convert_dict_structure

    Gelv2_dict = read_gff_file(Gelv2_gff)
    Yuan_dict = read_gff_file(Yuan_gff)

    Gelv2_gene_dict, Gelv2_chr_dict = convert_dict_structure(Gelv2_dict)
    Yuan_gene_dict, Yuan_chr_dict = convert_dict_structure(Yuan_dict)

    compare_chr_dict = gene_compare(
        Gelv2_chr_dict, Yuan_chr_dict, similarity_type='shorter_overlap_coverage', threshold=0.5)

    gelv2_list = []
    yuan_list = []
    for chr_id in compare_chr_dict:
        for strand in compare_chr_dict[chr_id]:
            for a, b in compare_chr_dict[chr_id][strand]:
                gelv2_list.append(a)
                yuan_list.append(b)

    gelv2_list = list(set(gelv2_list))
    yuan_list = list(set(yuan_list))

    # species job for merge Gel gene annotation

    Gelv2_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/jbrowse/data/Gel.genome.gff3'
    Yuan_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/yuan_pt_vs_xu/WPG_working/WPGmapper/yuan/genewise.best.gff'
    identity_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/identity.id'
    Gel_Pseudo_gff = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/Gel.Pseudo.gff3'
    tran_evidence_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v2.0/annotation_run/evm/T.gff3'

    from lib.common.genome.genome_feature2 import read_gff_file, get_mRNA_overlap, Gene
    from interlap import InterLap
    from api.xuyuxing.file_parser.fileIO import tsv_file_parse
    from lib.xuyuxing.math.set_operating import merge_intervals, overlap_between_interval_set, sum_interval_length

    tmp_info = tsv_file_parse(identity_file)

    skip_gene_list = {}
    skip_gene_x = {}
    for i in tmp_info:
        g_id = tmp_info[i][0]
        y_id = tmp_info[i][1]
        skip_gene_list[g_id] = None
        skip_gene_list[y_id] = None
        skip_gene_x[g_id] = None

    Gelv2_dict = read_gff_file(Gelv2_gff)
    Yuan_dict = read_gff_file(Yuan_gff)

    Gelv2_chr_dict = {}
    Gelv2_gene_dict = {}
    for i in Gelv2_dict:
        for gf_id in Gelv2_dict[i]:
            gf = Gelv2_dict[i][gf_id]
            gene = Gene(from_gf=gf)
            gene.build_gene_seq()

            if gene.chr_id not in Gelv2_chr_dict:
                Gelv2_chr_dict[gene.chr_id] = {"+": {}, "-": {}}

            Gelv2_chr_dict[gene.chr_id][gene.strand][gf_id] = gene
            Gelv2_gene_dict[gf_id] = gene

    Yuan_gene_dict = {}
    Yuan_chr_dict = {}
    for i in Yuan_dict:
        for gf_id in Yuan_dict[i]:
            gf = Yuan_dict[i][gf_id]
            gene = Gene(from_gf=gf)
            gene.build_gene_seq()

            if gene.chr_id not in Yuan_chr_dict:
                Yuan_chr_dict[gene.chr_id] = {"+": {}, "-": {}}

            Yuan_chr_dict[gene.chr_id][gene.strand][gf_id] = gene
            Yuan_gene_dict[gf_id] = gene

    gene_id_pair = []

    for chr_id in Gelv2_chr_dict:
        for strand in Gelv2_chr_dict[chr_id]:
            if chr_id in Yuan_chr_dict and strand in Yuan_chr_dict[chr_id]:
                num = 0
                for gf1_id in Gelv2_chr_dict[chr_id][strand]:
                    num += 1
                    print(chr_id, strand, num, len(
                        Gelv2_chr_dict[chr_id][strand]))
                    if gf1_id in skip_gene_list:
                        continue
                    for gf2_id in Yuan_chr_dict[chr_id][strand]:
                        gf2 = Yuan_gene_dict[gf2_id]
                        query_gf2 = gf2.qualifiers['query'][0]
                        if query_gf2 in skip_gene_list:
                            continue
                        overlap_flag = False
                        for m1 in Gelv2_chr_dict[chr_id][strand][gf1_id].sub_features:
                            if overlap_flag:
                                break
                            for m2 in Yuan_chr_dict[chr_id][strand][gf2_id].sub_features:
                                if overlap_flag:
                                    break
                                if get_mRNA_overlap(m1, m2, 'shorter_overlap_coverage') > 0.5:
                                    overlap_flag = True
                        if overlap_flag:
                            gene_id_pair.append((gf1_id, gf2_id))

    # tran
    tran_evidence_dict = {}
    tran_info_raw = tsv_file_parse(tran_evidence_gff)
    for i in tran_info_raw:
        q_name, strand, start, end = tran_info_raw[i][0], tran_info_raw[i][6], int(
            tran_info_raw[i][3]), int(tran_info_raw[i][4])
        if (q_name, strand) not in tran_evidence_dict:
            tran_evidence_dict[(q_name, strand)] = []
        tran_evidence_dict[(q_name, strand)].append((start, end))

    for i in tran_evidence_dict:
        merged_intervals = merge_intervals(tran_evidence_dict[i], True)
        tran_evidence_dict[i] = InterLap()
        if len(merged_intervals) > 0:
            tran_evidence_dict[i].update(merged_intervals)

    def get_tran_ratio(gf, tran_evidence_dict, type_exon='exon'):
        contig = gf.chr_id
        strand = gf.strand

        exon_intervals = [(i.start, i.end)
                          for i in gf.sub_features if i.type == type_exon]

        tran_intervals = list(
            tran_evidence_dict[(contig, strand)].find((gf.start, gf.end)))
        overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
            exon_intervals, tran_intervals)
        tran_overlap_ratio = overlap_length / \
            sum_interval_length(exon_intervals)

        return tran_overlap_ratio

    keep_x = []
    keep_y = []
    for x_id, y_id in gene_id_pair:
        x_gf = Gelv2_gene_dict[x_id]
        y_gf = Yuan_gene_dict[y_id]

        x_ratio = get_tran_ratio(
            x_gf.model_mRNA, tran_evidence_dict, type_exon='exon')
        y_ratio = get_tran_ratio(
            y_gf.sub_features[0], tran_evidence_dict, type_exon='CDS')

        print(x_ratio, y_ratio)

        if y_ratio - x_ratio > 0.2:
            keep_y.append(y_id)
        else:
            keep_x.append(x_id)

    keep_x = set(keep_x)
    keep_y = set(keep_y)

    for x_id, y_id in gene_id_pair:
        if x_id in keep_x and y_id in keep_y:
            keep_y.remove(y_id)

    all_compare_x = set([i[0] for i in gene_id_pair])
    remove_x = all_compare_x - keep_x
    not_compare_x = (set(list(Gelv2_gene_dict.keys())) - all_compare_x)

    all_compare_y = set([i[1] for i in gene_id_pair])
    not_compare_y = (set(list(Yuan_gene_dict.keys())) - all_compare_y)

    add_keep_y = []
    for i in not_compare_y:
        gf = Yuan_gene_dict[i]
        if not gf.qualifiers['query'][0] in skip_gene_list:
            add_keep_y.append(i)

    add_keep_x = []
    for i in not_compare_x:
        if not i in skip_gene_list:
            add_keep_x.append(i)

    good_gf_list = []

    for i in skip_gene_x:
        good_gf_list.append(Gelv2_gene_dict[i])
    for i in keep_x:
        good_gf_list.append(Gelv2_gene_dict[i])
    for i in keep_y:
        good_gf_list.append(Yuan_gene_dict[i])
    for i in add_keep_y:
        good_gf_list.append(Yuan_gene_dict[i])
    for i in add_keep_x:
        good_gf_list.append(Gelv2_gene_dict[i])

    ###

    Gel_Pseudo_dict = read_gff_file(Gel_Pseudo_gff)

    Gel_Pseudo_gene_dict = {}
    Gel_Pseudo_chr_dict = {}
    for i in Gel_Pseudo_dict:
        for gf_id in Gel_Pseudo_dict[i]:
            gf = Gel_Pseudo_dict[i][gf_id]
            gene = Gene(from_gf=gf)
            gene.build_gene_seq()

            if gene.chr_id not in Gel_Pseudo_chr_dict:
                Gel_Pseudo_chr_dict[gene.chr_id] = {"+": {}, "-": {}}

            Gel_Pseudo_chr_dict[gene.chr_id][gene.strand][gf_id] = gene
            Gel_Pseudo_gene_dict[gf_id] = gene

    good_gene_dict = {}
    good_chr_dict = {}
    for gf in good_gf_list:
        gene = Gene(from_gf=gf)
        gene.build_gene_seq()

        if gene.chr_id not in good_chr_dict:
            good_chr_dict[gene.chr_id] = {"+": {}, "-": {}}

        good_chr_dict[gene.chr_id][gene.strand][gf.id] = gene
        good_gene_dict[gf.id] = gene

    gene_id_pair = []

    for chr_id in Gel_Pseudo_chr_dict:
        for strand in Gel_Pseudo_chr_dict[chr_id]:
            if chr_id in good_chr_dict and strand in good_chr_dict[chr_id]:
                num = 0
                for gf1_id in Gel_Pseudo_chr_dict[chr_id][strand]:
                    num += 1
                    print(chr_id, strand, num, len(
                        Gel_Pseudo_chr_dict[chr_id][strand]))

                    for gf2_id in good_chr_dict[chr_id][strand]:

                        overlap_flag = False
                        for m1 in Gel_Pseudo_chr_dict[chr_id][strand][gf1_id].sub_features:
                            if overlap_flag:
                                break
                            for m2 in good_chr_dict[chr_id][strand][gf2_id].sub_features:
                                if overlap_flag:
                                    break
                                if get_mRNA_overlap(m1, m2, 'shorter_overlap_coverage') > 0.5:
                                    overlap_flag = True
                        if overlap_flag:
                            gene_id_pair.append((gf1_id, gf2_id))

    pass_pseudo_gene_list = set(
        Gel_Pseudo_gene_dict.keys()) - set([i[0] for i in gene_id_pair])

    # merge

    import re
    from lib.xuyuxing.base.common_command import merge_dict
    from lib.common.genome.genome_feature2 import gene_rename, write_gff_file
    import copy

    all_gf_list = []
    num = 0
    for contig in good_chr_dict:
        gf_dict = merge_dict(
            [good_chr_dict[contig]['+'], good_chr_dict[contig]['-']], False)
        sorted_gf_list = sorted(list(gf_dict.keys()),
                                key=lambda x: gf_dict[x].start)

        for i in sorted_gf_list:
            if i in Gelv2_gene_dict:
                all_gf_list.append(Gelv2_gene_dict[i])
            else:
                gf = Yuan_gene_dict[i]
                pseudo = 'Note' in gf.sub_features[0].qualifiers

                if pseudo:
                    new_gf_name = "GelREAP%05d" % num
                else:
                    new_gf_name = "GelREAG%05d" % num

                num = num + 1

                new_gf = gene_rename(gf, new_gf_name, gf.chr_id)

                if not pseudo:

                    cds_list = [
                        cds for cds in new_gf.sub_features[0].sub_features]

                    exon_list = []
                    for j in cds_list:
                        i = copy.deepcopy(j)
                        i.id = re.sub('cds', 'exon', i.id)
                        del i.qualifiers['phase']
                        i.qualifiers['ID'] = i.id
                        i.qualifiers['Name'] = i.id
                        i.type = 'exon'
                        exon_list.append(i)

                    new_gf.sub_features[0].sub_features.extend(exon_list)
                else:
                    new_gf.qualifiers['Note'] = 'pseudogene'

                all_gf_list.append(new_gf)

    for i in pass_pseudo_gene_list:
        gf = Gel_Pseudo_gene_dict[i]
        gf.qualifiers['Note'] = 'pseudogene'
        all_gf_list.append(gf)

    write_gff_file(
        all_gf_list, '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/merged.gff')

    # add fungi_pseudo_gene
    from lib.common.genome.genome_feature2 import read_gff_file, convert_dict_structure, get_mRNA_overlap

    fungi_pseudo_gff = "/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/pseudo/Gel.rep.gff3"
    Gelv3_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/v3.1/Gel.genome.v3.0.gff'

    fungi_pseudo_dict = read_gff_file(fungi_pseudo_gff)
    Gelv3_dict = read_gff_file(Gelv3_gff)

    fp_gene_dict, fp_chr_dict = convert_dict_structure(fungi_pseudo_dict)
    v3_gene_dict, v3_chr_dict = convert_dict_structure(Gelv3_dict)

    keep_fp_list = []
    num = 0
    for chr_id in fp_chr_dict:
        for strand in fp_chr_dict[chr_id]:
            if chr_id in v3_chr_dict and strand in v3_chr_dict[chr_id]:
                for fq_gf_id in fp_chr_dict[chr_id][strand]:
                    num += 1
                    print(num)
                    overlap_flag = False
                    for v3_gf_id in v3_chr_dict[chr_id][strand]:
                        if overlap_flag:
                            break
                        for m1 in fp_chr_dict[chr_id][strand][fq_gf_id].sub_features:
                            if overlap_flag:
                                break
                            for m2 in v3_chr_dict[chr_id][strand][v3_gf_id].sub_features:
                                if overlap_flag:
                                    break
                                if get_mRNA_overlap(m1, m2, 'shorter_overlap_coverage') > 0.5:
                                    overlap_flag = True
                    if not overlap_flag:
                        keep_fp_list.append(fq_gf_id)
            else:
                for fq_gf_id in fp_chr_dict[chr_id][strand]:
                    keep_fp_list.append(fq_gf_id)

    #
    import re
    from lib.common.genome.genome_feature2 import gene_rename, write_gff_file

    gf_list = []
    for i in v3_gene_dict:
        gf = v3_gene_dict[i]
        gf_list.append(gf)

    num = 1
    for i in keep_fp_list:
        gf = fp_gene_dict[i]
        new_gf_name = 'GelFGPS%05d' % num
        num += 1
        new_gf = gene_rename(gf, new_gf_name, gf.chr_id)
        new_gf.qualifiers['Note'] = 'pseudogene'
        gf_list.append(new_gf)

    Gelv3_1_gff = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/v3.1/Gel.genome.v3.1.gff'
    write_gff_file(gf_list, Gelv3_1_gff, "GelGenome", True)

    # test
    gff_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/v3.0/v3.1/Gel.genome.v3.1.gff'

    from lib.common.genome.genome_feature2 import read_gff_file, get_mRNA_overlap, Gene, gene_compare, convert_dict_structure

    Gelv31_dict = read_gff_file(gff_file)

    gene_dict, chr_dict = convert_dict_structure(Gelv31_dict)

    compare_chr_dict = gene_compare(
        chr_dict, chr_dict, similarity_type='shorter_overlap_coverage', threshold=0.5)

    pair_list = []
    for chr_id in compare_chr_dict:
        for strand in compare_chr_dict[chr_id]:
            for a, b in compare_chr_dict[chr_id][strand]:
                if a == b:
                    continue
                else:
                    pair_list.append((a, b))



"""
The script hopes to get the genomes which can represent all sequenced species for HGT study by analyzing
assembly_summary file ("assembly_summary_genbank.txt" and "assembly_summary_refseq.txt") and genome size file from NCBI.
ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank.txt
ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq.txt
ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/species_genome_size.txt.gz (need decompressed)
"""

__author__ = 'Yuxing Xu'

import re

import lib.common.os
import lib.xuyuxing.base.base_function as bf
from lib.xuyuxing.base.common_command import log_print
import time
import lib.common.sqlite_command as sc
import sqlite3
from functools import cmp_to_key
from lib.xuyuxing.evolution.taxonomy import read_tax_record_dict_db, read_tax_name_record_dict_db


class GenomeRecord(object):
    """
    Genome_record is a class for each record in NCBI genome database.
    """

    def __init__(self):
        pass


col_name = ["assembly_accession", "bioproject", "biosample", "wgs_master", "refseq_category", "taxid",
            "species_taxid", "organism_name", "infraspecific_name", "isolate", "version_status", "assembly_level",
            "release_type", "genome_rep", "seq_rel_date", "asm_name", "submitter", "gbrs_paired_asm",
            "paired_asm_comp", "ftp_path", "excluded_from_refseq", "relation_to_type_material"]


def build_ncbi_genome_database(assembly_summary_genbank_file, assembly_summary_refseq_file):
    """
    This func parse assembly_summary file and store data as dict which assembly_accession is keys
    and GenomeRecord class with column as attr is value.
    assembly_summary_genbank and assembly_summary_refseq have no overlap in level of assembly_accession
    :param assembly_summary_genbank_file:
    :param assembly_summary_refseq_file:
    :return: a dict
    """

    from api.xuyuxing.file_parser.fileIO import tsv_file_parse
    from lib.xuyuxing.base.common_command import dict2class
    parse_file_db = tsv_file_parse(assembly_summary_genbank_file)
    genbank_record_dict = dict2class(parse_file_db, GenomeRecord, col_name)
    parse_file_db = tsv_file_parse(assembly_summary_refseq_file)
    refseq_record_dict = dict2class(parse_file_db, GenomeRecord, col_name)

    record_dict = {}

    for i in genbank_record_dict.keys():
        record = genbank_record_dict[i]
        record.group = "genbank"
        record_dict[record.assembly_accession] = record

    for i in refseq_record_dict.keys():
        record = refseq_record_dict[i]
        record.group = "refseq"
        record_dict[record.assembly_accession] = record

    return record_dict


def genome_chooser(record_dict, tax_db_file, taxonomic_rank="genus", keep_no_rank=True, top_taxon=None):

    # step 1: Based on "gbrs_paired_asm" and "paired_asm_comp", I remove genbank assembly which paired with a refseq.
    #        Remove bad assembly that excluded_from_refseq is not empty.
    log_print("\t\tBegin: mark bad assembly record")
    for record in record_dict.keys():
        del_flag = 0
        if record_dict[record].group == "genbank" and record_dict[record].paired_asm_comp != "na":
            del_flag = 1
        if not record_dict[record].excluded_from_refseq == "":
            del_flag = 1
        if record_dict[record].ftp_path == "na":
            del_flag = 1
        if del_flag == 1:
            record_dict[record].del_flag = "Bad Record"
        else:
            record_dict[record].del_flag = "Good Record"
    log_print("\t\tEnd: mark bad assembly record")

    # step 2: Choose a best assembly to represent a given taxonomic_rank (like a genus)
    log_print("\t\tBegin: cluster assembly record to taxonomic rank")
    # classified assemblies by taxonomic_rank or lower rank(when given rank not in a record)
    class_taxon_rank_dict = {}
    num_genus = 0
    num_no_genus = 0
    if top_taxon:
        top_taxon_list = list(read_tax_name_record_dict_db(tax_db_file, [top_taxon]).values())
        if len(top_taxon_list) > 0:
            top_taxon_id = top_taxon_list[0].tax_id
        else:
            ValueError("Top taxon found nothing!")

    tax_record_db = read_tax_record_dict_db(tax_db_file,
                                            tuple([record_dict[accession_id].taxid for accession_id in record_dict]))
    num = 0
    start_time = time.time()
    for accession_id in record_dict.keys():
        record = record_dict[accession_id]
        if record.taxid in tax_record_db:
            taxid_taxon = tax_record_db[record.taxid]

            if top_taxon:
                if not top_taxon_id in set([i[0] for i in taxid_taxon.get_lineage]):
                    continue

            if hasattr(taxid_taxon, taxonomic_rank):
                num_genus = num_genus + 1
                taxonomic_rank_tmp = getattr(taxid_taxon, taxonomic_rank)
                setattr(record, taxonomic_rank, taxonomic_rank_tmp)
                if not taxonomic_rank_tmp in class_taxon_rank_dict:
                    class_taxon_rank_dict[taxonomic_rank_tmp] = []
                class_taxon_rank_dict[taxonomic_rank_tmp].append(record)
            else:
                if keep_no_rank:
                    num_no_genus = num_no_genus + 1
                    setattr(record, taxonomic_rank, "None")
                    if not record.taxid in class_taxon_rank_dict:
                        class_taxon_rank_dict[record.taxid] = []
                    class_taxon_rank_dict[record.taxid].append(record)

        num = num + 1
        round_time = time.time()
        if round_time - start_time > 10:
            log_print("\t\t\t%d/%d" % (num, len(record_dict)))
            start_time = round_time

    # sort by sort_dict
    def cmp_tmp(x, y):
        key_rank = ["del_flag", "group", "refseq_category",
                    "assembly_level", "genome_rep"]

        sort_dict = {
            "del_flag": {"Good Record": 1, "Bad Record": 2},
            "group": {"refseq": 1, "genbank": 2},
            "refseq_category": {"reference genome": 1, "representative genome": 2, "na": 3},
            "assembly_level": {"Chromosome": 1, "Complete Genome": 2, "Scaffold": 3, "Contig": 4},
            "genome_rep": {"Full": 1, "Partial": 2},
        }

        compara_flag = 0
        for key_tmp in key_rank:
            x_key_tmp = getattr(x, key_tmp)
            y_key_tmp = getattr(y, key_tmp)
            if x_key_tmp == y_key_tmp:
                continue
            elif sort_dict[key_tmp][x_key_tmp] > sort_dict[key_tmp][y_key_tmp]:
                compara_flag = 1
                break
            elif sort_dict[key_tmp][x_key_tmp] < sort_dict[key_tmp][y_key_tmp]:
                compara_flag = -1
                break

        return compara_flag

    sorted_class_taxon_rank_dict = {}
    for each_taxon in class_taxon_rank_dict:
        assembly_list = class_taxon_rank_dict[each_taxon]
        sorted_assembly_list = sorted(assembly_list, key=cmp_to_key(cmp_tmp))
        sorted_class_taxon_rank_dict[each_taxon] = sorted_assembly_list

    # def print_assembly_list(assembly_list,tax_id):
    #    key_rank = ["group", "refseq_category", "assembly_level", "genome_rep"]
    #    for i in assembly_list:
    #        printer = tax_id+"\t"
    #        for j in key_rank:
    #            printer = printer + getattr(i, j) + ";"
    #        print printer
    #
    # for each_taxon in sorted_class_taxon_rank_dict:
    #    print_assembly_list(sorted_class_taxon_rank_dict[each_taxon],each_taxon)

    log_print("\t\tEnd: cluster assembly record to taxonomic rank")

    return sorted_class_taxon_rank_dict


def store_sorted_class_taxon_rank_dict_into_sqlite(sorted_class_taxon_rank_dict, db_file, taxonomic_rank):
    column_name = ["taxon_group_id", "grouped", "Rank", "del_flag"] + col_name
    sc.init_sql_db(db_file, "ncbi_assembly", column_name)
    record_tmp_list = []
    num = 0
    for each_taxon in sorted_class_taxon_rank_dict:
        assembly_list = sorted_class_taxon_rank_dict[each_taxon]
        rank_num = 0
        for record in assembly_list:
            record_tmp = []
            num = num + 1
            # print(num)
            rank_num = rank_num + 1
            # taxon_group_id
            record_tmp.append(each_taxon)
            # grouped
            if not getattr(record, taxonomic_rank) == "None":
                record_tmp.append("Yes")
            else:
                record_tmp.append("No")
            # Rank
            record_tmp.append(rank_num)
            # del_flag
            record_tmp.append(record.del_flag)
            # other
            for i in col_name:
                record_tmp.append(getattr(record, i).replace("\"", ""))
            record_tmp_list.append(tuple(record_tmp))

            if len(record_tmp_list) % 10000 == 0:
                sc.sqlite_write(record_tmp_list, db_file,
                                "ncbi_assembly", column_name)
                record_tmp_list = []

    if len(record_tmp_list) > 0:
        sc.sqlite_write(record_tmp_list, db_file, "ncbi_assembly", column_name)
        record_tmp_list = []

    conn = sqlite3.connect(db_file)
    conn.execute(
        "CREATE UNIQUE INDEX assembly_accession_index on ncbi_assembly (assembly_accession)")
    conn.close()


def read_choosed_GenomeRecord_from_sqlite(db_file, taxonomic_rank):
    column_name = ["taxon_group_id", "grouped", "Rank", "del_flag"] + col_name

    conn = sqlite3.connect(db_file)
    table_name = "ncbi_assembly"
    record_list = conn.execute(
        "select * from ncbi_assembly WHERE Rank = 1 AND del_flag = \"Good Record\"").fetchall()
    conn.close()

    record_dict = {}
    for record in record_list:
        record_tmp = GenomeRecord()
        if record[1] == "Yes":
            setattr(record_tmp, taxonomic_rank, record[0])
        else:
            setattr(record_tmp, taxonomic_rank, "None")

        record_tmp.del_flag = record[3]

        num = 0
        for i in record[4:]:
            setattr(record_tmp, col_name[num], i)
            num = num + 1

        record_dict[record[0]] = record_tmp

    return record_dict


def print_sorted_class_taxon_rank_dict_from_sqlite(db_file, txt_file):
    import csv

    column_name = ["taxon_group_id", "grouped", "Rank", "del_flag"] + col_name

    with open(txt_file, 'w', newline='') as csvfile:
        spamwriter = csv.writer(csvfile, delimiter=',',
                                quotechar='"', quoting=csv.QUOTE_MINIMAL)
        spamwriter.writerow(column_name)

        conn = sqlite3.connect(db_file)
        table_name = "ncbi_assembly"
        record_list = conn.execute("select * from ncbi_assembly").fetchall()
        conn.close()

        for record in record_list:
            spamwriter.writerow(record)


def ascp_download(online_path, down_path, printer=False):
    from config import ascp_path
    cmd_string = ascp_path + " -v -i ~/.aspera/connect/etc/asperaweb_id_dsa.openssh -k 1 -T -l 200m anonftp@ftp-private.ncbi.nlm.nih.gov:%s %s" % (
        online_path, down_path)

    if printer is True:
        return cmd_string

    flag, output, error = lib.common.os.cmd_run(cmd_string)
    # print(flag, output, error)
    if re.match(".*Completed.*", str(output)):
        return True
    else:
        return False


def wget_download(online_path, down_path, printer=False):
    from config import wget_path
    # cmd_string = wget_path + " -t 0 -O %s ftp://ftp.ncbi.nlm.nih.gov%s" % (
    #     down_path, online_path)
    cmd_string = wget_path + " -t 0 -O %s %s" % (
        down_path, online_path)

    print(cmd_string)

    if printer is True:
        return cmd_string

    flag, output, error = lib.common.os.cmd_run(cmd_string, silence=True)

    return True


def md5checksums_checker(para_dict_tmp, ncbi_record_md5_renew_flag, download_way='ascp'):
    import os

    if download_way == 'ascp':
        download_function = ascp_download
    elif download_way == 'wget':
        download_function = wget_download

    # search exist file
    if not os.path.exists(para_dict_tmp["data_dir_tmp"]):
        lib.common.os.cmd_run(
            "mkdir " + para_dict_tmp["data_dir_tmp"], silence=True)

    # print(record_tmp.assembly_accession)
    md5_file_url, md5_file_path = para_dict_tmp["files"]["md5_file"]
    if ncbi_record_md5_renew_flag is False:
        if os.path.exists(md5_file_path) and os.path.getsize(md5_file_path) != 0:
            return os.path.getsize(md5_file_path)
        else:
            if download_function(md5_file_url, md5_file_path):
                return os.path.getsize(md5_file_path)
            else:
                raise EnvironmentError("Check network!")

    if ncbi_record_md5_renew_flag is True:
        if os.path.exists(md5_file_path):
            lib.common.os.cmd_run("rm " + md5_file_path, silence=True)
        if download_function(md5_file_url, md5_file_path):
            return os.path.getsize(md5_file_path)
        else:
            raise EnvironmentError("Check network!")

    raise EnvironmentError("Unknown error!")


def genome_download(record_tmp, root_dir, want_download):
    """
    want_download = ["gff_file", "pt_file", "cds_file", "genome_file"]
    :param record_tmp:
    :param root_dir:
    :param want_download:
    :return:
    """
    from lib.common.os import md5sum_check
    from api.xuyuxing.file_parser.fileIO import tsv_file_parse
    data_dir = root_dir + "/data/NCBI"

    # parameter access
    ftp_path = record_tmp.ftp_path
    ftp_dir_path = ftp_path.replace("ftp://ftp.ncbi.nlm.nih.gov", "")
    data_dir_tmp = data_dir + "/" + \
        re.search("(^\S+\_\d+)\.\d+", record_tmp.assembly_accession).group(1)
    para_dict = {
        "speci_accession": re.search("(^\S+\_\d+)\.\d+", record_tmp.assembly_accession).group(1),
        "assembly_accession": record_tmp.assembly_accession,
        "full_accession": ftp_path.split("/")[-1],
        "gff_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_genomic.gff.gz",
                     data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_genomic.gff.gz"),
        "pt_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_translated_cds.faa.gz",
                    data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_translated_cds.faa.gz"),
        "cds_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_cds_from_genomic.fna.gz",
                     data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_cds_from_genomic.fna.gz"),
        "genome_file": (ftp_dir_path + "/" + ftp_path.split("/")[-1] + "_genomic.fna.gz",
                        data_dir_tmp + "/" + ftp_path.split("/")[-1] + "_genomic.fna.gz"),
        "md5_file": (ftp_dir_path + "/" + "md5checksums.txt", data_dir_tmp + "/" + "md5checksums.txt")
    }

    # # add md5 info
    md5_list = tsv_file_parse(para_dict["md5_file"][1], seq=r' ')
    for i in want_download:
        for j in md5_list:
            md5_string, nothing, file_name = md5_list[j]
            if para_dict[i][0].split("/")[-1] == file_name.split("/")[-1]:
                para_dict[i] = (para_dict[i][0], para_dict[i][1], md5_string)

    # # find uncompleted file
    log_dict = {}
    for i in want_download:
        if len(para_dict[i]) == 3:
            online_path, file_path, md5_string = para_dict[i]
            if not md5sum_check(file_path, md5_string):
                log_dict[i] = "renew"
            else:
                log_dict[i] = "ok"
        else:
            log_dict[i] = "miss"

    # # download
    for i in log_dict:
        if log_dict[i] == "renew":
            online_path, file_path, md5_string = para_dict[i]
            if ascp_download(online_path, file_path) and md5sum_check(file_path, md5_string):
                print("Renew %s" % i)
                log_dict[i] = "ok"

    related_path = "/data/NCBI"
    for i in log_dict:
        if log_dict[i] == "ok":
            online_path, file_path, md5_string = para_dict[i]
            log_dict[i] = related_path + "/" + \
                para_dict["speci_accession"] + "/" + file_path.split("/")[-1]

    return log_dict


if __name__ == '__main__':
    """
    The script hopes to get the genomes which can represent all sequenced species for HGT study by analyzing
    assembly_summary file ("assembly_summary_genbank.txt" and "assembly_summary_refseq.txt") and genome size file from NCBI.
    ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_genbank.txt
    ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/assembly_summary_refseq.txt
    ftp://ftp.ncbi.nlm.nih.gov/genomes/ASSEMBLY_REPORTS/species_genome_size.txt.gz (need decompressed)
    """
    from lib.xuyuxing.evolution.taxonomy import build_taxon_database, store_tax_record_into_sqlite

    work_dir = '/lustre/home/xuyuxing/Database/NCBI/genome'

    # input file
    assembly_summary_genbank_file = work_dir + '/assembly_summary_genbank.txt'
    assembly_summary_refseq_file = work_dir + '/assembly_summary_refseq.txt'

    # taxonomy
    path_for_taxonomy = "/lustre/home/xuyuxing/Database/NCBI/taxonomy"
    tax_db_file = path_for_taxonomy + "/tax_xyx.db"
    # tax_record_dict = build_taxon_database(path_for_taxonomy)
    # store_tax_record_into_sqlite(tax_record_dict, tax_db_file)

    # output file
    ncbi_choosed_record_txt_file = work_dir + "/assembly_choosed.csv"

    # parse ncbi
    record_dict = build_ncbi_genome_database(
        assembly_summary_genbank_file, assembly_summary_refseq_file)
    ncbi_genus_record = genome_chooser(
        record_dict, tax_db_file, taxonomic_rank='genus')



from Bio import SeqIO
import gzip
from lib.common.sqlite_command import pickle_dump_obj, build_database, build_index, sqlite_select, pickle_load_obj
import sqlite3
from api.xuyuxing.file_parser.xml_parser import getelements
from lib.xuyuxing.evolution.taxonomy import read_tax_record_dict_db

def store_uniprot_swiss_into_sqlite(xml_file, sql3_db_file, gzip_flag=False):

    table_columns_dict = {'record': ['id', 'codenstring']}

    if gzip_flag:
        handle = gzip.open(xml_file, "rt")
    else:
        handle = xml_file

    data_generator = SeqIO.parse(handle, "uniprot-xml")

    def parse_function(seqio_obj):
        return {'record': (seqio_obj.id, pickle_dump_obj(seqio_obj))}

    build_database(data_generator, parse_function,
                   table_columns_dict, sql3_db_file)

    build_index(sql3_db_file, 'record', 'id')

    if gzip_flag:
        handle.close()

    return sql3_db_file


def extract_swiss_info(swiss_id_list, sql3_db_file):
    data_get = sqlite_select(sql3_db_file, 'record', [
                             'id', 'codenstring'], key_name='id', value_tuple=tuple(swiss_id_list))
    output_dict = {}
    for id_tmp, codenstring in data_get:
        obj = pickle_load_obj(codenstring)
        output_dict[id_tmp] = obj
    return output_dict


def iterUniprotDB(db_file):
    conn = sqlite3.connect(db_file)
    for id_tmp, codenstring in conn.execute('SELECT * FROM record'):
        obj = pickle_load_obj(codenstring)
        yield (id_tmp, obj)
    conn.close()



def filter_by_taxon(input_dict, target_taxon_id, taxon_db_file):
    # input_dict = {id: [id, seq, taxon_list]}

    all_taxon_id_list = []
    for i in input_dict:
        taxon_list = input_dict[i][2]
        all_taxon_id_list.extend(taxon_list)

    all_taxon_id_list = list(set(all_taxon_id_list))

    tax_dict = read_tax_record_dict_db(taxon_db_file, tax_id_list=all_taxon_id_list)

    output_dict = {}
    for i in input_dict:
        save_flag = False
        for j in input_dict[i][2]:
            if j in tax_dict:
                t = tax_dict[j]
                t_ancestor = set([i[0] for i in t.get_lineage])
                if target_taxon_id in t_ancestor:
                    save_flag = True

        if save_flag:
            output_dict[i] = input_dict[i]

    return output_dict

def get_plant_uniprot90(uniprot90_xml,taxon_db_file,output_file):
    # 3398 Magnoliopsida angiosperms flowering plants
    plant = '3398'

    with open(output_file, 'w') as f:
        tmp_store_dict = {}
        input_handle = gzip.open(uniprot90_xml, 'r')
        num = 0
        for entry in getelements(input_handle,'{http://uniprot.org/uniref}entry'):
            num += 1
            

            entry_id = entry.attrib['id']

            representativeMember_list = entry.findall('{http://uniprot.org/uniref}representativeMember')
            rep_member = representativeMember_list[0]
            rep_seq = rep_member.find('{http://uniprot.org/uniref}sequence').text

            dbReference = rep_member.find('{http://uniprot.org/uniref}dbReference')

            taxon_id_list = []
            for property_tmp in dbReference:
                if property_tmp.attrib['type'] == "NCBI taxonomy":
                    taxon_id = property_tmp.attrib['value']
                    taxon_id_list.append(taxon_id)

            member_list = entry.findall('{http://uniprot.org/uniref}member')

            for member in member_list:

                dbReference = member.find('{http://uniprot.org/uniref}dbReference')

                for property_tmp in dbReference:
                    if property_tmp.attrib['type'] == "NCBI taxonomy":
                        taxon_id = property_tmp.attrib['value']
                        taxon_id_list.append(taxon_id)

            taxon_id_list = list(set(taxon_id_list))

            tmp_store_dict[entry_id] = [entry_id, rep_seq, taxon_id_list]

            if len(tmp_store_dict) > 50000:
                print(num)
                output_dict = filter_by_taxon(tmp_store_dict, plant, taxon_db_file)
                tmp_store_dict = {}

                for r_id in output_dict:
                    f.write(">%s\n%s\n" % (r_id, output_dict[r_id][1]))

        output_dict = filter_by_taxon(tmp_store_dict, plant, taxon_db_file)

        for r_id in output_dict:
            f.write(">%s\n%s\n" % (r_id, output_dict[r_id][1]))    

                

                

if __name__ == "__main__":

    # # store xml into sqlite

    # xml_file = "/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_sprot_plants.xml.gz"
    # db_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_sprot_plants.db'
    # store_uniprot_swiss_into_sqlite(xml_file, db_file, True)

    # xml_file = "/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_trembl_plants.xml.gz"
    # db_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_trembl_plants.db'
    # store_uniprot_swiss_into_sqlite(xml_file, db_file, True)


    # # iter database
    # db_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_sprot_plants.db'
    # db_fasta_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_sprot_plants.faa'

    # with open(db_fasta_file, 'w') as f:
    #     for id_tmp, obj in iterUniprotDB(db_file):
    #         f.write(">%s\n%s\n" % (id_tmp, str(obj.seq)))

    # db_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_trembl_plants.db'
    # db_fasta_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/plant/uniprot_trembl_plants.faa'

    # with open(db_fasta_file, 'w') as f:
    #     for id_tmp, obj in iterUniprotDB(db_file):
    #         f.write(">%s\n%s\n" % (id_tmp, str(obj.seq)))

    # # extract record by accession

    # swiss_id_list = ['W8QP19', 'W8QNH8']

    # extract_swiss_info(swiss_id_list, db_file)

    # #
    taxon_db_file = '/lustre/home/xuyuxing/Database/NCBI/taxonomy/tax_xyx.db'
    uniprot90_xml = '/lustre/home/xuyuxing/Database/Uniprot/2020/uniref50.xml.gz'
    output_file = '/lustre/home/xuyuxing/Database/Uniprot/2020/uniref50.plant.faa'

    get_plant_uniprot90(uniprot90_xml,taxon_db_file,output_file)